@@ -1,6 +1,6 @@
-from starlite.asgi.routing_trie.mapping import add_map_route_to_trie
+from starlite.asgi.routing_trie.mapping import add_route_to_trie
 from starlite.asgi.routing_trie.traversal import parse_scope_to_route
 from starlite.asgi.routing_trie.types import RouteTrieNode
 from starlite.asgi.routing_trie.validate import validate_node
 
-__all__ = ["RouteTrieNode", "add_map_route_to_trie", "parse_scope_to_route", "validate_node"]
+__all__ = ["RouteTrieNode", "add_route_to_trie", "parse_scope_to_route", "validate_node"]
