@@ -2,10 +2,12 @@ from typing import Any
 
 
 class StarLiteException(Exception):
+    """Base exception class from which all Starlite exceptions inherit."""
+
     detail: str
 
     def __init__(self, *args: Any, detail: str = "") -> None:
-        """Base exception class from which all Starlite exceptions inherit.
+        """Initialize `StarLiteException`.
 
         Args:
             *args (Any): args are cast to `str` before passing to `Exception.__init__()`
@@ -26,6 +28,5 @@ class StarLiteException(Exception):
 class MissingDependencyException(StarLiteException):
     """Missing optional dependency.
 
-    This exception is raised only when a module depends on a dependency
-    that has not been installed.
+    This exception is raised only when a module depends on a dependency that has not been installed.
     """
