@@ -1,9 +1,12 @@
+from collections import defaultdict
+from functools import lru_cache
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
+    DefaultDict,
     Dict,
     List,
-    Mapping,
     NamedTuple,
     Optional,
     Set,
@@ -32,13 +35,12 @@ from starlite.constants import (
     EXTRA_KEY_REQUIRED,
     RESERVED_KWARGS,
 )
-from starlite.datastructures import URL
 from starlite.datastructures.provide import Provide
 from starlite.enums import ParamType, RequestEncodingType
 from starlite.exceptions import ImproperlyConfiguredException, ValidationException
-from starlite.parsers import parse_form_data
+from starlite.parsers import parse_form_data, parse_query_string
 from starlite.signature import SignatureModel, get_signature_model
-from starlite.utils.dependency import resolve_dependencies_concurrently
+from starlite.types import Empty
 
 if TYPE_CHECKING:
     from starlite.connection import Request, WebSocket
@@ -87,6 +89,70 @@ def merge_parameter_sets(first: Set[ParameterDefinition], second: Set[ParameterD
     return result
 
 
+def create_connection_value_extractor(
+    connection_key: str,
+    expected_params: Set[ParameterDefinition],
+    parser: Optional[Callable[[Union["WebSocket", "Request"]], Dict[str, Any]]] = None,
+) -> Callable[[Dict[str, Any], Union["WebSocket", "Request"]], None]:
+    """Create a kwargs extractor function.
+
+    Args:
+        connection_key: The attribute key to use.
+        expected_params: The set of expected params.
+        parser: An optional parser function.
+
+    Returns:
+        An extractor function.
+    """
+
+    alias_and_key_tuple = tuple(
+        (p.field_alias.lower() if p.param_type == ParamType.HEADER else p.field_alias, p.field_name)
+        for p in expected_params
+    )
+    alias_defaults = {
+        p.field_alias.lower() if p.param_type == ParamType.HEADER else p.field_alias: p.default_value
+        for p in expected_params
+        if not (p.is_required or p.default_value is Ellipsis)
+    }
+
+    def extractor(values: Dict[str, Any], connection: Union["WebSocket", "Request"]) -> None:
+        data = parser(connection) if parser else getattr(connection, connection_key, {})
+
+        try:
+            connection_mapping: Dict[str, Any] = {
+                key: data[alias] if alias in data else alias_defaults[alias] for alias, key in alias_and_key_tuple
+            }
+            values.update(connection_mapping)
+        except KeyError as e:
+            raise ValidationException(f"Missing required parameter {e.args[0]} for url {connection.url}") from e
+
+    return extractor
+
+
+@lru_cache
+def create_query_default_dict(
+    parsed_query: Tuple[Tuple[str, str], ...], sequence_query_parameter_names: Tuple[str, ...]
+) -> DefaultDict[str, Union[List[str], str]]:
+    """Transform a list of tuples into a default dict. Ensures non-list values are not wrapped in a list.
+
+    Args:
+        parsed_query: The parsed query list of tuples.
+        sequence_query_parameter_names: A set of query parameters that should be wrapped in list.
+
+    Returns:
+        A default dict
+    """
+    output: DefaultDict[str, Union[List[str], str]] = defaultdict(list)
+
+    for k, v in parsed_query:
+        if k in sequence_query_parameter_names:
+            output[k].append(v)  # type: ignore
+        else:
+            output[k] = v
+
+    return output
+
+
 class KwargsModel:
     """Model required kwargs for a given RouteHandler and its dependencies.
 
@@ -101,6 +167,7 @@ class KwargsModel:
         "expected_path_params",
         "expected_query_params",
         "expected_reserved_kwargs",
+        "extractors",
         "has_kwargs",
         "is_data_optional",
         "sequence_query_parameter_names",
@@ -139,7 +206,8 @@ class KwargsModel:
         self.expected_path_params = expected_path_params
         self.expected_query_params = expected_query_params
         self.expected_reserved_kwargs = expected_reserved_kwargs
-        self.sequence_query_parameter_names = sequence_query_parameter_names
+        self.sequence_query_parameter_names = tuple(sequence_query_parameter_names)
+
         self.has_kwargs = (
             expected_cookie_params
             or expected_dependencies
@@ -149,7 +217,39 @@ class KwargsModel:
             or expected_query_params
             or expected_reserved_kwargs
         )
+
         self.is_data_optional = is_data_optional
+        self.extractors = self._create_extractors()
+
+    def _create_extractors(self) -> List[Callable[[Dict[str, Any], Union["WebSocket", "Request"]], None]]:
+        extractors: List[Callable[[Dict[str, Any], Union["WebSocket", "Request"]], None]] = []
+        if self.expected_header_params:
+            extractors.append(
+                create_connection_value_extractor(
+                    connection_key="headers", expected_params=self.expected_header_params
+                ),
+            )
+        if self.expected_path_params:
+            extractors.append(
+                create_connection_value_extractor(
+                    connection_key="path_params", expected_params=self.expected_path_params
+                ),
+            )
+        if self.expected_cookie_params:
+            extractors.append(
+                create_connection_value_extractor(
+                    connection_key="cookies", expected_params=self.expected_cookie_params
+                ),
+            )
+        if self.expected_query_params:
+            extractors.append(
+                create_connection_value_extractor(
+                    connection_key="query_params",
+                    expected_params=self.expected_query_params,
+                    parser=self._parse_connection_query_params,
+                ),
+            )
+        return extractors
 
     @classmethod
     def _get_param_definitions(
@@ -333,14 +433,11 @@ class KwargsModel:
             else False,
         )
 
-    def _collect_reserved_kwargs(
-        self, connection: Union["WebSocket", "Request"], connection_query_params: Dict[str, Union[str, List[str]]]
-    ) -> Dict[str, Any]:
+    def _collect_reserved_kwargs(self, connection: Union["WebSocket", "Request"]) -> Dict[str, Any]:
         """Create and populate dictionary of "reserved" keyword arguments.
 
         Args:
             connection: An instance of [Request][starlite.connection.Request] or [WebSocket][starlite.connection.WebSocket].
-            connection_query_params: The query params dct.
 
         Returns:
             A dictionary of values correlating to reserved kwargs.
@@ -353,7 +450,7 @@ class KwargsModel:
         if "cookies" in self.expected_reserved_kwargs:
             reserved_kwargs["cookies"] = connection.cookies
         if "query" in self.expected_reserved_kwargs:
-            reserved_kwargs["query"] = connection_query_params
+            reserved_kwargs["query"] = connection.query_params
         if "request" in self.expected_reserved_kwargs:
             reserved_kwargs["request"] = connection
         if "socket" in self.expected_reserved_kwargs:
@@ -364,6 +461,24 @@ class KwargsModel:
             reserved_kwargs["scope"] = connection.scope
         return reserved_kwargs
 
+    def _parse_connection_query_params(self, connection: Union["WebSocket", "Request"]) -> Dict[str, Any]:
+        """Parse query params and cache the result in scope.
+
+        Args:
+            connection: The ASGI connection instance.
+
+        Returns:
+            A dictionary of parsed values.
+        """
+        parsed_query = connection.scope["_parsed_query"] = (  # type: ignore
+            connection._parsed_query  # pylint: disable=protected-access
+            if connection._parsed_query is not Empty  # pylint: disable=protected-access
+            else parse_query_string(connection.scope.get("query_string", b""))
+        )
+        return create_query_default_dict(
+            parsed_query=parsed_query, sequence_query_parameter_names=self.sequence_query_parameter_names
+        )
+
     def to_kwargs(self, connection: Union["WebSocket", "Request"]) -> Dict[str, Any]:
         """Return a dictionary of kwargs. Async values, i.e. CoRoutines, are not resolved to ensure this function is
         sync.
@@ -374,49 +489,15 @@ class KwargsModel:
         Returns:
             A string keyed dictionary of kwargs expected by the handler function and its dependencies.
         """
-        connection_query_params = {
-            k: self._sequence_or_scalar_param(k, v) for k, v in connection.query_params.dict().items()
-        }
+        output: Dict[str, Any] = {}
 
-        path_params = self._collect_params(
-            params=connection.path_params, expected=self.expected_path_params, url=connection.url
-        )
-        query_params = self._collect_params(
-            params=connection_query_params, expected=self.expected_query_params, url=connection.url
-        )
-        header_params = self._collect_params(
-            params=connection.headers, expected=self.expected_header_params, url=connection.url
-        )
-        cookie_params = self._collect_params(
-            params=connection.cookies, expected=self.expected_cookie_params, url=connection.url
-        )
+        for extractor in self.extractors:
+            extractor(output, connection)
 
-        if not self.expected_reserved_kwargs:
-            return {**path_params, **query_params, **header_params, **cookie_params}
-        return {
-            **self._collect_reserved_kwargs(connection=connection, connection_query_params=connection_query_params),
-            **path_params,
-            **query_params,
-            **header_params,
-            **cookie_params,
-        }
+        if self.expected_reserved_kwargs:
+            output.update(self._collect_reserved_kwargs(connection=connection))
 
-    @staticmethod
-    def _collect_params(params: Mapping[str, Any], expected: Set[ParameterDefinition], url: URL) -> Dict[str, Any]:
-        """Collect request params, checking for missing required values."""
-        missing_params = [p.field_alias for p in expected if p.is_required and p.field_alias not in params]
-        if missing_params:
-            raise ValidationException(f"Missing required parameter(s) {', '.join(missing_params)} for url {url}")
-        return {p.field_name: params.get(p.field_alias, p.default_value) for p in expected}
-
-    async def _resolve_dependency(
-        self, dependency: "Dependency", connection: Union["WebSocket", "Request"], **kwargs: Any
-    ) -> Any:
-        """Recursively resolve the dependency graph."""
-        signature_model = get_signature_model(dependency.provide)
-        await resolve_dependencies_concurrently(self, dependency.dependencies, connection, kwargs)
-        dependency_kwargs = signature_model.parse_values_from_connection_kwargs(connection=connection, **kwargs)
-        return await dependency.provide(**dependency_kwargs)
+        return output
 
     async def resolve_dependency(
         self, dependency: "Dependency", connection: Union["WebSocket", "Request"], **kwargs: Any
@@ -431,17 +512,13 @@ class KwargsModel:
         Returns:
             The resolved dependency value
         """
-        if dependency.provide.cache_per_request:
-            async with dependency.provide.lock:
-                cache = connection.state.setdefault("_dependency_cache", {})
-                if dependency.provide.cache_key in cache:
-                    return cache[dependency.provide.cache_key]
-
-                value = await self._resolve_dependency(dependency, connection, **kwargs)
-                cache[dependency.provide.cache_key] = value
-                return value
-        else:
-            return await self._resolve_dependency(dependency, connection, **kwargs)
+        signature_model = get_signature_model(dependency.provide)
+        for sub_dependency in dependency.dependencies:
+            kwargs[sub_dependency.key] = await self.resolve_dependency(
+                dependency=sub_dependency, connection=connection, **kwargs
+            )
+        dependency_kwargs = signature_model.parse_values_from_connection_kwargs(connection=connection, **kwargs)
+        return await dependency.provide(**dependency_kwargs)
 
     @classmethod
     def _create_dependency_graph(cls, key: str, dependencies: Dict[str, Provide]) -> Dependency:
@@ -552,12 +629,6 @@ class KwargsModel:
                 f"The following kwargs have been used: {', '.join(used_reserved_kwargs)}"
             )
 
-    def _sequence_or_scalar_param(self, key: str, value: List[str]) -> Union[str, List[str]]:
-        """Return the first element of 'value' if we expect it to be a scalar value (appears in
-        self.sequence_query_parameter_names) and it contains only a single element.
-        """
-        return value[0] if key not in self.sequence_query_parameter_names and len(value) == 1 else value
-
     async def _get_request_data(self, request: "Request") -> Any:
         """Retrieve the data - either json data or form data - from the request"""
         if self.expected_form_data:
